@@ -1,6 +1,8 @@
-/*  mailcheck.c
+/*  $Id: mailcheck.c,v 1.48 2004/09/19 09:29:16 jasper Exp $
  *
- *  Copyright (C) 2002 Jasper Huijsmans (huysmans@users.sourceforge.net)
+ *  Copyright (C) 2002-2004 Jasper Huijsmans (jasper@xfce.org)
+ *
+ *  - 2004 Add Maildir support Julien NOEL (dev@no-l.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,17 +30,26 @@
 #include <sys/wait.h>
 #endif
 
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+
 #include <stdio.h>
 
 #include <gtk/gtk.h>
 
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/debug.h>
+#include <netdb.h>
+#include <sys/socket.h>
+
+#include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/xfce_iconbutton.h>
 
 #include <panel/xfce.h>
 #include <panel/settings.h>
 #include <panel/plugins.h>
+#include <panel/item_dialog.h>
+
+#include <dirent.h>
 
 #define MAILCHECK_ROOT "Mailcheck"
 
@@ -50,10 +61,16 @@ typedef struct
 {
     char *mbox;
     char *command;
+    char *newmail_command;
     gboolean term;
     gboolean use_sn;
     int interval;
 
+    gboolean pop3;
+    char pop3_username[256];
+    char pop3_password[256];
+    char pop3_hostname[256];
+
     int timeout_id;
     int status;
 
@@ -76,25 +93,52 @@ enum
 };
 
 static char *mailcheck_icon_names[] = {
-    "nomail",
-    "newmail",
-    "oldmail"
+    "xfce-mail",
+    "mail",
+    "gnome-mailcheck",
+    NULL
 };
 
 static GdkPixbuf *
-get_mailcheck_pixbuf (int id)
+get_mail_pixbuf (void)
 {
     GdkPixbuf *pb;
-    char *name = mailcheck_icon_names[id];
 
-    pb = get_themed_pixbuf (name);
+    pb = themed_pixbuf_from_name_list (mailcheck_icon_names,
+				       icon_size[settings.size]);
 
-    if (!pb || !GDK_IS_PIXBUF (pb))
+    if (G_UNLIKELY (!pb))
+    {
+	g_critical ("Mail icon could not be found");
 	pb = get_pixbuf_by_id (UNKNOWN_ICON);
+    }
 
     return pb;
 }
 
+static void
+reset_mailcheck_icons (t_mailcheck * mc)
+{
+    if (mc->newmail_pb)
+	g_object_unref (mc->newmail_pb);
+
+    if (mc->nomail_pb)
+	g_object_unref (mc->nomail_pb);
+
+    if (mc->oldmail_pb)
+	g_object_unref (mc->oldmail_pb);
+
+    mc->newmail_pb = get_mail_pixbuf ();
+
+    g_return_if_fail (mc->newmail_pb != NULL);
+
+    mc->nomail_pb = gdk_pixbuf_copy (mc->newmail_pb);
+    gdk_pixbuf_saturate_and_pixelate (mc->nomail_pb, mc->nomail_pb, 0, TRUE);
+
+    mc->oldmail_pb = mc->nomail_pb;
+    g_object_ref (mc->oldmail_pb);
+}
+
 /*  Configuration
  *  -------------
 */
@@ -106,7 +150,7 @@ mailcheck_set_tip (t_mailcheck * mc)
     if (!tooltips)
 	tooltips = gtk_tooltips_new ();
 
-    if (mc->command && strlen(mc->command))
+    if (mc->command && strlen (mc->command))
     {
 	tip = g_strdup (mc->command);
 
@@ -117,19 +161,159 @@ mailcheck_set_tip (t_mailcheck * mc)
 	g_free (tip);
     }
 }
-    
+
 static gboolean
 set_mail_icon (t_mailcheck * mc)
 {
     if (mc->status == NO_MAIL)
+    {
 	xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (mc->button),
 				    mc->nomail_pb);
+    }
     else if (mc->status == OLD_MAIL)
+    {
 	xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (mc->button),
 				    mc->oldmail_pb);
+    }
     else
+    {
 	xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (mc->button),
 				    mc->newmail_pb);
+    }
+
+    return FALSE;
+}
+
+/* POP3 routines */
+
+static int
+pop3_read_response (int fd, char *buff, int size)
+{
+    int bytes_read = 0;
+    int state = 0;
+
+    while (bytes_read < size && state != 2)
+    {
+	bytes_read += read (fd, &buff[bytes_read], sizeof (char));
+
+	switch (state)
+	{
+	    case 0:
+		if (buff[bytes_read - 1] == '\r')
+		    state = 1;
+		break;
+	    case 1:
+		if (buff[bytes_read - 1] == '\n')
+		    state = 2;
+		else
+		    state = 0;
+		break;
+	}
+    }
+
+    if (state == 2)
+	buff[bytes_read - 2 * sizeof (char)] = '\0';
+
+    if (strncmp (buff, "+OK", 3 * sizeof (char)) == 0)
+	return 1;
+    else
+	return 0;
+}
+
+static int
+pop3_send_command (int fd, char *buff)
+{
+    return write (fd, buff, strlen (buff));
+}
+
+static int
+pop3_check_mail (char *username, char *password, char *hostname)
+{
+    struct sockaddr_in sa;
+    struct hostent *hp;
+    int msg_count;
+    char buff[1024];
+    char command[256];
+    int port = 110;
+    int sd;
+    int i;
+
+    /* Lookup hostname */
+    hp = gethostbyname (hostname);
+    if (hp == NULL)
+	return NO_MAIL;
+
+    sa.sin_port = htons (port);
+    sa.sin_family = hp->h_addrtype;
+
+    /* Create the socket */
+    sd = socket (AF_INET, SOCK_STREAM, 0);
+    if (sd == -1)
+	return NO_MAIL;
+
+    /* Try each address until we get one that works */
+    for (i = 0;; i++)
+    {
+	if (hp->h_addr_list[i] == NULL)
+	{
+	    close (sd);
+	    return NO_MAIL;
+	}
+
+	memcpy ((char *) &sa.sin_addr, hp->h_addr_list[i], hp->h_length);
+
+	if (connect (sd, (struct sockaddr *) &sa, sizeof (sa)) == 0)
+	    break;
+    }
+
+    if (!pop3_read_response (sd, buff, 1024))
+    {
+	close (sd);
+	return NO_MAIL;
+    }
+
+    g_snprintf (command, 256, "USER %s\r\n", username);
+    pop3_send_command (sd, command);
+    if (!pop3_read_response (sd, buff, 1024))
+    {
+	pop3_send_command (sd, "QUIT\r\n");
+	close (sd);
+	return NO_MAIL;
+    }
+
+    g_snprintf (command, 256, "PASS %s\r\n", password);
+    pop3_send_command (sd, command);
+    if (!pop3_read_response (sd, buff, 1024))
+    {
+	pop3_send_command (sd, "QUIT\r\n");
+	close (sd);
+	return NO_MAIL;
+    }
+
+    pop3_send_command (sd, "STAT\r\n");
+    if (!pop3_read_response (sd, buff, 1024))
+    {
+	pop3_send_command (sd, "QUIT\r\n");
+	close (sd);
+	return NO_MAIL;
+    }
+
+    sscanf (buff, "+OK %d", &msg_count);
+
+    pop3_send_command (sd, "QUIT\r\n");
+
+    close (sd);
+
+    if (msg_count > 0)
+	return NEW_MAIL;
+    else
+	return NO_MAIL;
+}
+
+static gboolean
+exec_newmail_cmd (const char *cmd)
+{
+    exec_cmd (cmd, FALSE, FALSE);
 
     return FALSE;
 }
@@ -137,29 +321,103 @@ set_mail_icon (t_mailcheck * mc)
 static gboolean
 check_mail (t_mailcheck * mailcheck)
 {
-    int mail;
+    int mail = NO_MAIL;
     struct stat s;
 
-    DBG("Checking mail ... ");
-    
-    if (stat (mailcheck->mbox, &s) < 0)
-	mail = NO_MAIL;
-    else if (!s.st_size)
-	mail = NO_MAIL;
-    else if (s.st_mtime <= s.st_atime)
-	mail = OLD_MAIL;
+    DBG ("Checking mail ... ");
+
+    if (mailcheck->pop3 == TRUE)
+    {
+	mail = pop3_check_mail (mailcheck->pop3_username,
+				mailcheck->pop3_password,
+				mailcheck->pop3_hostname);
+    }
     else
-	mail = NEW_MAIL;
+    {
+	if (stat (mailcheck->mbox, &s) < 0)
+	{
+	    mail = NO_MAIL;
+	}
+	else if (S_ISREG (s.st_mode))
+	{
+	    DBG ("mbox format");
+
+	    if (!s.st_size)
+		mail = NO_MAIL;
+	    else if (s.st_mtime <= s.st_atime)
+		mail = OLD_MAIL;
+	    else
+		mail = NEW_MAIL;
+
+	}
+	else if (S_ISDIR (s.st_mode))
+	{
+	    DIR *dr;
+	    struct dirent *de;
+	    char *c_list[] = { "tmp", "cur", "new" };
+	    char *c_tmp;
+	    int i_tmp;
+
+	    /* mailbox is a maildir */
+	    DBG ("maildir format");
+
+	    mail = NO_MAIL;
+
+	    /* Verify the Maildir integrity */
+	    for (i_tmp = 0; i_tmp < 3; i_tmp++)
+	    {
+		c_tmp =
+		    g_build_filename (mailcheck->mbox, c_list[i_tmp], NULL);
+
+		if (stat (c_tmp, &s) >= 0 && S_ISDIR (s.st_mode))
+		{
+		    dr = opendir (c_tmp);
+
+		    if (dr != NULL)
+		    {
+			while ((de = readdir (dr)))
+			{
+			    if (strlen (de->d_name) >= 1 &&
+				de->d_name[0] != '.')
+			    {
+				if (i_tmp < 2)
+				{
+				    mail = OLD_MAIL;
+				    break;
+				}
+				else
+				{
+				    mail = NEW_MAIL;
+				    break;
+				}
+			    }
+			}
+
+			closedir (dr);
+		    }
+		}
+	    }
+	}
+    }
+
+    DBG ("Done");
 
     if (mail != mailcheck->status)
     {
-	mailcheck->status = mail;
+	if (mail == NEW_MAIL && mailcheck->status != NEW_MAIL &&
+	    mailcheck->newmail_command &&
+	    strlen (mailcheck->newmail_command) != 0)
+	{
+	    g_idle_add ((GSourceFunc) exec_newmail_cmd,
+			mailcheck->newmail_command);
+	}
 
+	mailcheck->status = mail;
 	g_idle_add ((GSourceFunc) set_mail_icon, mailcheck);
     }
 
-    DBG("Done\n");
-    
+    DBG ("Done\n");
+
     /* keep the g_timeout running */
     return TRUE;
 }
@@ -169,10 +427,7 @@ static void
 run_mailcheck (t_mailcheck * mc)
 {
     if (mc->timeout_id > 0)
-    {
-	g_source_remove (mc->timeout_id);
-	mc->timeout_id = 0;
-    }
+	return;
 
     if (mc->interval > 0)
     {
@@ -181,8 +436,7 @@ run_mailcheck (t_mailcheck * mc)
     }
 }
 
-
-void
+static void
 mailcheck_read_config (Control * control, xmlNodePtr node)
 {
     xmlChar *value;
@@ -215,11 +469,29 @@ mailcheck_read_config (Control * control, xmlNodePtr node)
 	if (xmlStrEqual (node->name, (const xmlChar *) "Mbox"))
 	{
 	    value = DATA (node);
-
 	    if (value)
 	    {
 		g_free (mc->mbox);
 		mc->mbox = (char *) value;
+
+		if (strncmp (mc->mbox, "pop3://", 7 * sizeof (char)) == 0)
+		{
+		    mc->pop3 = TRUE;
+		    sscanf (mc->mbox, "pop3://%[^:]:%[^@]@%s",
+			    mc->pop3_username,
+			    mc->pop3_password, mc->pop3_hostname);
+		}
+	    }
+	}
+	else if (xmlStrEqual
+		 (node->name, (const xmlChar *) "newmail-command"))
+	{
+	    value = DATA (node);
+
+	    if (value)
+	    {
+		g_free (mc->newmail_command);
+		mc->newmail_command = (char *) value;
 	    }
 	}
 	else if (xmlStrEqual (node->name, (const xmlChar *) "Command"))
@@ -245,6 +517,7 @@ mailcheck_read_config (Control * control, xmlNodePtr node)
 
 		g_free (value);
 	    }
+
 	    value = xmlGetProp (node, "sn");
 
 	    if (value)
@@ -266,7 +539,7 @@ mailcheck_read_config (Control * control, xmlNodePtr node)
     mailcheck_set_tip (mc);
 }
 
-void
+static void
 mailcheck_write_config (Control * control, xmlNodePtr parent)
 {
     xmlNodePtr root, node;
@@ -283,11 +556,13 @@ mailcheck_write_config (Control * control, xmlNodePtr parent)
 
     node = xmlNewTextChild (root, NULL, "Command", mc->command);
 
-    snprintf (value, 2, "%d", mc->term);
+    g_snprintf (value, 2, "%d", mc->term);
     xmlSetProp (node, "term", value);
 
-    snprintf (value, 2, "%d", mc->use_sn);
+    g_snprintf (value, 2, "%d", mc->use_sn);
     xmlSetProp (node, "sn", value);
+
+    xmlNewTextChild (root, NULL, "newmail-command", mc->newmail_command);
 }
 
 static void
@@ -317,14 +592,16 @@ mailcheck_new (void)
     mailcheck->interval = 30;
     mailcheck->timeout_id = 0;
 
-    mailcheck->nomail_pb = get_mailcheck_pixbuf (NO_MAIL);
-    mailcheck->oldmail_pb = get_mailcheck_pixbuf (OLD_MAIL);
-    mailcheck->newmail_pb = get_mailcheck_pixbuf (NEW_MAIL);
+    reset_mailcheck_icons (mailcheck);
+
+    mailcheck->newmail_command = g_strdup ("");
 
     mail = g_getenv ("MAIL");
 
     if (mail)
+    {
 	mailcheck->mbox = g_strdup (mail);
+    }
     else
     {
 	const char *logname = g_getenv ("LOGNAME");
@@ -346,7 +623,7 @@ mailcheck_new (void)
     return mailcheck;
 }
 
-void
+static void
 mailcheck_free (Control * control)
 {
     t_mailcheck *mailcheck = (t_mailcheck *) control->data;
@@ -356,6 +633,7 @@ mailcheck_free (Control * control)
 
     g_free (mailcheck->mbox);
     g_free (mailcheck->command);
+    g_free (mailcheck->newmail_command);
 
     g_object_unref (mailcheck->nomail_pb);
     g_object_unref (mailcheck->oldmail_pb);
@@ -364,31 +642,28 @@ mailcheck_free (Control * control)
     g_free (mailcheck);
 }
 
-
-
-
 static void
 mailcheck_set_theme (Control * control, const char *theme)
 {
     t_mailcheck *mailcheck = (t_mailcheck *) control->data;
 
-    g_object_unref (mailcheck->nomail_pb);
-    g_object_unref (mailcheck->oldmail_pb);
-    g_object_unref (mailcheck->newmail_pb);
-
-    mailcheck->nomail_pb = get_mailcheck_pixbuf (NO_MAIL);
-    mailcheck->oldmail_pb = get_mailcheck_pixbuf (OLD_MAIL);
-    mailcheck->newmail_pb = get_mailcheck_pixbuf (NEW_MAIL);
+    reset_mailcheck_icons (mailcheck);
 
     if (mailcheck->status == NO_MAIL)
+    {
 	xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (mailcheck->button),
 				    mailcheck->nomail_pb);
+    }
     else if (mailcheck->status == OLD_MAIL)
+    {
 	xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (mailcheck->button),
 				    mailcheck->oldmail_pb);
+    }
     else
+    {
 	xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (mailcheck->button),
 				    mailcheck->newmail_pb);
+    }
 }
 
 /*  Options dialog
@@ -398,44 +673,20 @@ typedef struct
 {
     t_mailcheck *mc;
 
-    /* backup */
-    char *mbox;
-    char *command;
-    gboolean term;
-    gboolean use_sn;
-    int interval;
-
     /* control dialog */
     GtkWidget *dialog;
 
     /* options */
     GtkWidget *mbox_entry;
-    GtkWidget *cmd_entry;
-    GtkWidget *term_cb;
-    GtkWidget *sn_cb;
     GtkWidget *interval_spin;
+    GtkWidget *newmail_cmd_entry;
+    CommandOptions *cmd_opts;
 }
 MailDialog;
 
-/* backup */
-static void
-create_backup (MailDialog * md)
-{
-    t_mailcheck *mc = md->mc;
-
-    md->mbox = g_strdup (mc->mbox);
-    md->command = g_strdup (mc->command);
-    md->term = mc->term;
-    md->use_sn = mc->use_sn;
-    md->interval = mc->interval;
-}
-
 static void
 free_maildialog (MailDialog * md)
 {
-    g_free (md->mbox);
-    g_free (md->command);
-
     g_free (md);
 }
 
@@ -446,17 +697,10 @@ mailcheck_apply_options (MailDialog * md)
     const char *tmp;
     t_mailcheck *mc = md->mc;
 
-    tmp = gtk_entry_get_text (GTK_ENTRY (md->cmd_entry));
+    g_free (mc->command);
 
-    if (tmp && *tmp)
-    {
-	g_free (mc->command);
-	mc->command = g_strdup (tmp);
-    }
-
-    mc->term = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (md->term_cb));
-
-    mc->use_sn = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (md->sn_cb));
+    command_options_get_command (md->cmd_opts, &(mc->command), &(mc->term),
+				 &(mc->use_sn));
 
     tmp = gtk_entry_get_text (GTK_ENTRY (md->mbox_entry));
 
@@ -466,6 +710,14 @@ mailcheck_apply_options (MailDialog * md)
 	mc->mbox = g_strdup (tmp);
     }
 
+    tmp = gtk_entry_get_text (GTK_ENTRY (md->newmail_cmd_entry));
+
+    if (tmp && *tmp)
+    {
+	g_free (mc->newmail_command);
+	mc->newmail_command = g_strdup (tmp);
+    }
+
     mc->interval =
 	gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
 					  (md->interval_spin));
@@ -475,30 +727,6 @@ mailcheck_apply_options (MailDialog * md)
     run_mailcheck (mc);
 }
 
-#if 0
-/* restore baclup */
-static void
-mailcheck_revert_options (MailDialog * md)
-{
-    if (md->mbox)
-	gtk_entry_set_text (GTK_ENTRY (md->mbox_entry), md->mbox);
-
-    if (md->command)
-	gtk_entry_set_text (GTK_ENTRY (md->cmd_entry), md->command);
-
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (md->term_cb), md->term);
-
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (md->sn_cb), md->use_sn);
-
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON (md->interval_spin),
-			       md->interval);
-
-    mailcheck_apply_options (md);
-
-    gtk_widget_set_sensitive (md->revert, FALSE);
-}
-#endif
-
 /* mbox */
 static void
 mbox_browse_cb (GtkWidget * b, MailDialog * md)
@@ -518,8 +746,27 @@ mbox_browse_cb (GtkWidget * b, MailDialog * md)
     }
 }
 
-gboolean
-mbox_entry_lost_focus (MailDialog * md)
+/* newmail command */
+static void
+newmail_cmd_brows_cb (GtkWidget * b, MailDialog * md)
+{
+    const char *text;
+    char *file;
+
+    text = gtk_entry_get_text (GTK_ENTRY (md->newmail_cmd_entry));
+
+    file = select_file_name (NULL, text, md->dialog);
+
+    if (file)
+    {
+	gtk_entry_set_text (GTK_ENTRY (md->newmail_cmd_entry), file);
+	g_free (file);
+	mailcheck_apply_options (md);
+    }
+}
+
+static gboolean
+entry_lost_focus (MailDialog * md)
 {
     mailcheck_apply_options (md);
 
@@ -531,6 +778,7 @@ static void
 add_mbox_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
 {
     GtkWidget *hbox, *label, *button, *image;
+    t_mailcheck *mc = md->mc;
 
     hbox = gtk_hbox_new (FALSE, BORDER);
     gtk_widget_show (hbox);
@@ -543,148 +791,82 @@ add_mbox_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
     md->mbox_entry = gtk_entry_new ();
-    if (md->mbox)
-	gtk_entry_set_text (GTK_ENTRY (md->mbox_entry), md->mbox);
+    if (mc->mbox)
+	gtk_entry_set_text (GTK_ENTRY (md->mbox_entry), mc->mbox);
     gtk_widget_show (md->mbox_entry);
     gtk_box_pack_start (GTK_BOX (hbox), md->mbox_entry, TRUE, TRUE, 0);
 
-    button = gtk_button_new();
+    button = gtk_button_new ();
     gtk_widget_show (button);
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
-    image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
-    gtk_widget_show(image);
-    gtk_container_add(GTK_CONTAINER(button), image);
-    
+    image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
+    gtk_widget_show (image);
+    gtk_container_add (GTK_CONTAINER (button), image);
+
     g_signal_connect (button, "clicked", G_CALLBACK (mbox_browse_cb), md);
 
     /* only set label on focus out */
     g_signal_connect_swapped (md->mbox_entry, "focus-out-event",
-			      G_CALLBACK (mbox_entry_lost_focus), md);
-}
-
-/* command */
-static void
-command_browse_cb (GtkWidget * b, MailDialog * md)
-{
-    const char *text;
-    char *file;
-
-    text = gtk_entry_get_text (GTK_ENTRY (md->cmd_entry));
-
-    file = select_file_name (NULL, text, md->dialog);
-
-    if (file)
-    {
-	gtk_entry_set_text (GTK_ENTRY (md->cmd_entry), file);
-	g_free (file);
-/*	mailcheck_apply_options(md);*/
-    }
-}
-
-gboolean
-command_entry_lost_focus (MailDialog * md)
-{
-    const char *tmp;
-
-    tmp = gtk_entry_get_text (GTK_ENTRY (md->cmd_entry));
-
-    if (tmp && *tmp)
-    {
-	g_free (md->mc->command);
-	md->mc->command = g_strdup (tmp);
-    }
-
-    /* needed to let entry handle focus-out as well */
-    return FALSE;
+			      G_CALLBACK (entry_lost_focus), md);
 }
 
 static void
-term_toggled (GtkToggleButton * tb, MailDialog * md)
+add_newmail_cmd_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
 {
-    md->mc->term = gtk_toggle_button_get_active (tb);
-}
-
-static void
-sn_toggled (GtkToggleButton * tb, MailDialog * md)
-{
-    md->mc->use_sn = gtk_toggle_button_get_active (tb);
-}
-
-static void
-add_command_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
-{
-    GtkWidget *hbox, *hbox2, *vbox2, *label, *button, *align, *image;
+    GtkWidget *hbox, *label, *button, *image;
+    t_mailcheck *mc = md->mc;
 
     hbox = gtk_hbox_new (FALSE, BORDER);
     gtk_widget_show (hbox);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-    label = gtk_label_new (_("Mail command:"));
+    label = gtk_label_new (_("New mail command:"));
     gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
     gtk_size_group_add_widget (sg, label);
     gtk_widget_show (label);
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-    md->cmd_entry = gtk_entry_new ();
-    if (md->command)
-	gtk_entry_set_text (GTK_ENTRY (md->cmd_entry), md->command);
-    gtk_widget_show (md->cmd_entry);
-    gtk_box_pack_start (GTK_BOX (hbox), md->cmd_entry, TRUE, TRUE, 0);
+    md->newmail_cmd_entry = gtk_entry_new ();
+    if (mc->newmail_command)
+	gtk_entry_set_text (GTK_ENTRY (md->newmail_cmd_entry),
+			    mc->newmail_command);
+    gtk_widget_show (md->newmail_cmd_entry);
+    gtk_box_pack_start (GTK_BOX (hbox), md->newmail_cmd_entry, TRUE, TRUE, 0);
 
-    gtk_tooltips_set_tip (tooltips, md->cmd_entry,
-			  _
-			  ("Command to run when the button on the panel is clicked"),
-			  NULL);
+    gtk_tooltips_set_tip (tooltips, md->newmail_cmd_entry,
+			  _("Command to run when new mail arrives"), NULL);
 
-    button = gtk_button_new();
+    button = gtk_button_new ();
     gtk_widget_show (button);
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
-    image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
-    gtk_widget_show(image);
-    gtk_container_add(GTK_CONTAINER(button), image);
-    
-    g_signal_connect (button, "clicked", G_CALLBACK (command_browse_cb), md);
+    image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
+    gtk_widget_show (image);
+    gtk_container_add (GTK_CONTAINER (button), image);
 
-    /* only set command on focus out */
-    g_signal_connect_swapped (md->cmd_entry, "focus-out-event",
-			      G_CALLBACK (command_entry_lost_focus), md);
+    g_signal_connect (button, "clicked", G_CALLBACK (newmail_cmd_brows_cb),
+		      md);
 
-    /* run in terminal ? */
-    hbox2 = gtk_hbox_new (FALSE, BORDER);
-    gtk_widget_show (hbox2);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
+    /* only set label on focus out */
+    g_signal_connect_swapped (md->mbox_entry, "focus-out-event",
+			      G_CALLBACK (entry_lost_focus), md);
+}
 
-    align = gtk_alignment_new (0, 0, 0, 0);
-    gtk_widget_show (align);
-    gtk_size_group_add_widget (sg, align);
-    gtk_box_pack_start (GTK_BOX (hbox2), align, FALSE, FALSE, 0);
+/* command */
+static void
+add_command_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
+{
+    md->cmd_opts = create_command_options (sg);
 
-    vbox2 = gtk_vbox_new (FALSE, BORDER);
-    gtk_widget_show (vbox2);
-    gtk_box_pack_start (GTK_BOX (hbox2), vbox2, FALSE, FALSE, 0);
+    command_options_set_command (md->cmd_opts, md->mc->command,
+				 md->mc->term, md->mc->use_sn);
 
-    md->term_cb = gtk_check_button_new_with_mnemonic (_("Run in _terminal"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (md->term_cb), md->term);
-    gtk_widget_show (md->term_cb);
-    gtk_box_pack_start (GTK_BOX (vbox2), md->term_cb, FALSE, FALSE, 0);
-
-    g_signal_connect (md->term_cb, "toggled", G_CALLBACK (term_toggled), md);
-
-    md->sn_cb =
-	gtk_check_button_new_with_mnemonic (_("Use startup _notification"));
-    gtk_widget_show (md->sn_cb);
-    gtk_box_pack_start (GTK_BOX (vbox2), md->sn_cb, FALSE, FALSE, 0);
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (md->sn_cb), md->use_sn);
-    gtk_widget_set_sensitive (md->sn_cb, TRUE);
-#else
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (md->sn_cb), FALSE);
-    gtk_widget_set_sensitive (md->sn_cb, FALSE);
-#endif
-    g_signal_connect (md->sn_cb, "toggled", G_CALLBACK (sn_toggled), md);
+    gtk_box_pack_start (GTK_BOX (vbox), md->cmd_opts->base, FALSE, TRUE, 0);
 
+    gtk_tooltips_set_tip (tooltips, md->cmd_opts->command_entry,
+			  _("Command to run when the button "
+			    "on the panel is clicked"), NULL);
 }
 
 /* interval */
@@ -698,6 +880,7 @@ static void
 add_interval_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
 {
     GtkWidget *hbox, *label;
+    t_mailcheck *mc = md->mc;
 
     hbox = gtk_hbox_new (FALSE, BORDER);
     gtk_widget_show (hbox);
@@ -711,7 +894,7 @@ add_interval_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
 
     md->interval_spin = gtk_spin_button_new_with_range (1, 600, 1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (md->interval_spin),
-			       md->interval);
+			       mc->interval);
     gtk_widget_show (md->interval_spin);
     gtk_box_pack_start (GTK_BOX (hbox), md->interval_spin, FALSE, FALSE, 0);
 
@@ -720,31 +903,33 @@ add_interval_box (GtkWidget * vbox, GtkSizeGroup * sg, MailDialog * md)
 }
 
 /* the dialog */
-void
-mailcheck_add_options (Control * control, GtkContainer * container,
-		       GtkWidget * done)
+static void
+mailcheck_create_options (Control * control, GtkContainer * container,
+			  GtkWidget * done)
 {
     GtkWidget *vbox;
     GtkSizeGroup *sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
     MailDialog *md;
 
+    xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
+
     md = g_new0 (MailDialog, 1);
 
     md->mc = (t_mailcheck *) control->data;
 
     md->dialog = gtk_widget_get_toplevel (done);
 
-    create_backup (md);
-
-    vbox = gtk_vbox_new (TRUE, BORDER);
+    vbox = gtk_vbox_new (FALSE, BORDER);
     gtk_widget_show (vbox);
 
     add_mbox_box (vbox, sg, md);
 
-    add_command_box (vbox, sg, md);
-
     add_interval_box (vbox, sg, md);
 
+    add_newmail_cmd_box (vbox, sg, md);
+
+    add_command_box (vbox, sg, md);
+
     /* signals */
     g_signal_connect_swapped (done, "clicked",
 			      G_CALLBACK (mailcheck_apply_options), md);
@@ -756,7 +941,7 @@ mailcheck_add_options (Control * control, GtkContainer * container,
 }
 
 /* create panel control */
-gboolean
+static gboolean
 create_mailcheck_control (Control * control)
 {
     t_mailcheck *mailcheck = mailcheck_new ();
@@ -774,8 +959,10 @@ create_mailcheck_control (Control * control)
 G_MODULE_EXPORT void
 xfce_control_class_init (ControlClass * cc)
 {
+    xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
+
     cc->name = "mailcheck";
-    cc->caption = _("Mail check");
+    cc->caption = _("Mail checker");
 
     cc->create_control = (CreateControlFunc) create_mailcheck_control;
 
@@ -784,7 +971,7 @@ xfce_control_class_init (ControlClass * cc)
     cc->write_config = mailcheck_write_config;
     cc->attach_callback = mailcheck_attach_callback;
 
-    cc->add_options = (gpointer) mailcheck_add_options;
+    cc->create_options = (gpointer) mailcheck_create_options;
 
     cc->set_theme = mailcheck_set_theme;
 }
